import torch
import torch.nn as nn
import torch.nn.functional as F


class Normalize(nn.Module):

    def __init__(self, power=2):
        super(Normalize, self).__init__()
        self.power = power

    def forward(self, x):
        norm = x.pow(self.power).sum(1, keepdim=True).pow(1./self.power)
        out = x.div(norm)
        return out


class Projector(nn.Module):
    def __init__(self, expansion=0):
        super(Projector, self).__init__()

        self.linear_1 = nn.Linear(512*expansion, 2048)
        self.linear_2 = nn.Linear(2048, 128)
        self.normalize = Normalize(2)

    def forward(self, x):

        output = self.linear_1(x)
        output = F.relu(output)

        output = self.linear_2(output)
        output = self.normalize(output)

        return output
